package converters;

import readers.*;
import writers.*;

import tests.Test;

import java.io.File;

import java.util.Vector;

public final class DocumentsConverter {
    
    private DocumentsConverter() {
        
        mInputFormats = new Vector< String >();
        mOutputFormats = new Vector< String >();
        mInputReaders = new Vector< TestReader > ();
        mOutputWriters = new Vector< TestWriter >();
        mInputEncodings = new Vector< String >();
        mOutputEncodings = new Vector< String >();
        
        mInputFormats.add( new String( "XML" ) );
        mInputReaders.add( new XMLTestReader() );
        mInputFormats.add( new String( "TXT ( Format Alpha )" ) );
        mInputReaders.add( new TXTAlphaTestReader() );
        
        mOutputFormats.add( new String( "XML" ) );
        mOutputWriters.add( new XMLTestWriter() );
        mOutputFormats.add( new String( "TXT ( Format Beta )" ) );
        mOutputWriters.add( new TXTBetaTestWriter() );
        
        mInputEncodings.add( new String( "UTF-8" ) );
        mInputEncodings.add( new String( "Cp1251" ) );
        
        mOutputEncodings.add( new String( "UTF-8" ) );
        mOutputEncodings.add( new String( "Cp1251" ) );
    }
    
    public static DocumentsConverter getInstance() {
        
        return mInstance;
    }
    
    public Vector< String > getInputFormats() {
        
        return ( Vector< String > ) mInputFormats.clone();
    }
    
    public Vector< String > getOutputFormats() {
        
        return ( Vector< String > ) mOutputFormats.clone();
    }
    
    public Vector< String > getInputEncodings() {
        
        return ( Vector< String > ) mInputEncodings.clone();
    }
    
    public Vector< String > getOutputEncodings() {
        
        return ( Vector< String > ) mOutputEncodings.clone();
    }
    
    public Vector< String > build(
            Integer pInputFormat,
            Integer pOutputFormat,
            Integer pInputEncoding,
            Integer pOutputEncoding ) {
        
        Vector< String > lLog = new Vector< String >();
        
        TestReader lReader = null;
        TestWriter lWriter = null;
        if( ( pInputFormat >= 0 ) && ( pInputFormat < mInputFormats.size() ) ) {
            lLog.add( "Source file format: " + mInputFormats.get( pInputFormat ) + "." );
            lReader = mInputReaders.get( pInputFormat );
            lReader.setSource( new File( mSourceFile ) );
        } else {
            lLog.add( "Unknown source file format." );
            return lLog;
        }
        
        if( ( pOutputFormat >= 0 ) && ( pOutputFormat < mOutputFormats.size() ) ) {
            lLog.add( "Destination file format: " + mOutputFormats.get( pOutputFormat ) + "." );
            lWriter = mOutputWriters.get( pOutputFormat );
            lWriter.setDestination( new File( mDestinationFile ) );
        } else {
            lLog.add( "Unknown destination file format." );
            return lLog;
        }
        
        if( ( pInputEncoding >= 0 ) && ( pInputEncoding < mInputEncodings.size() ) ) {
            lLog.add( "Source file encoding: " + mInputEncodings.get( pInputEncoding ) + "." );
            lReader.setEncoding( mInputEncodings.get( pInputEncoding ) );
        } else {
            lLog.add( "Unknown source file encoding." );
            return lLog;
        }
        
        if( ( pOutputEncoding >= 0 ) && ( pOutputEncoding < mOutputEncodings.size() ) ) {
            lLog.add( "Destination file encoding: " + mOutputEncodings.get( pOutputEncoding ) + "." );
            lWriter.setEncoding( mOutputEncodings.get( pOutputEncoding ) );
        } else {
            lLog.add( "Unknown destination file encoding." );
            return lLog;
        }
        
        Test lTest = lReader.read();
        if( lTest == null ) {
            Vector< String > lReadLog = lReader.getLog();
            lLog.addAll( lReadLog );
            return lLog;
        } else {
            lLog.add( "Reading complete successfully." );
        }
        
        if( lWriter.write( lTest ) ) {
            lLog.add( "Writing complete successfully." );
        } else {
            Vector< String > lWriteLog = lWriter.getLog();
            lLog.addAll( lWriteLog );
            return lLog;
        }
        
        return lLog;
    }
    
    public void setSourceFile(
            String pSourceFile ) {
        
        mSourceFile = pSourceFile;
    }
    
    public void setDestinationFile(
            String pDestinationFile ) {
        
        mDestinationFile = pDestinationFile;
    }
    
    private static DocumentsConverter mInstance = new DocumentsConverter();
    
    private Vector< String > mInputFormats;
    private Vector< String > mOutputFormats;
    
    private Vector< String > mInputEncodings;
    private Vector< String > mOutputEncodings;
    
    private Vector< TestReader > mInputReaders;
    private Vector< TestWriter > mOutputWriters;
    
    private String mSourceFile;
    private String mDestinationFile;
}