package readers;

import tests.*;

import java.io.File;

import org.w3c.dom.*;
import javax.xml.parsers.*;

import java.util.Vector;

import java.util.Date;

public class XMLTestReader
    extends FileTestReader {
    
    public XMLTestReader() {
        
        mXMLDocument = null;
    }
    
    public Test read() {
        
        if( ! ready() ) {
            return null;
        }
        
        Element lRoot = mXMLDocument.getDocumentElement();
        NodeList lTestNodes = lRoot.getChildNodes();
        
        Test lTest = null;
        mLog = new Vector< String >();
        try {
            lTest = parseTest( lTestNodes );
        } catch ( Exception eNotWellFormedXMLDocument ) {
            mLog.add( "Source XML file is not valid." );
        }
        
        return lTest;
    }
    
    public boolean ready() {
        
        if( super.ready() ) {
            createXMLDocument();
            if( mXMLDocument != null ) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
    
    private void createXMLDocument() {
        
        mXMLDocument = null;
        DocumentBuilderFactory lFactory = DocumentBuilderFactory.newInstance();
        try {
            DocumentBuilder lBuilder = lFactory.newDocumentBuilder();
            mXMLDocument = lBuilder.parse( mSourceFile );
        } catch (Exception eFailed ) {
            mLog.add( "Source XML file is not well-formed." );
        }
    }
    
    private Test parseTest(
            NodeList pTestNodes )
            throws Exception {
        
        String lSubject = null;
        String lTitle = null;
        Vector <Author> lAuthors = null;
        Author lEditor = null;
        Author lReviewer = null;
        Vector< Question > lQuestions = new Vector <Question>();
        Date lCreation = null;
        Date lPublication = null;
        Date lRenovation = null;
        Description lDescription = null;
        
        Test lTest = null;
        
        for( int iNode = 0; iNode < pTestNodes.getLength(); iNode++ ) {
            Node lNode = pTestNodes.item( iNode );
            if( lNode instanceof Element ) {
                Element lElement = ( Element )lNode;
                if( lElement.getTagName().equalsIgnoreCase( "subject" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lSubject = new String( lTextElement.getData().trim() );
                    if( lSubject.trim().length() == 0 ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->subject: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->subject: " + "empty." );
                    }
                } else if( lElement.getTagName().equalsIgnoreCase( "title" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lTitle = new String( lTextElement.getData().trim() );
                    if( lTitle.trim().length() == 0 ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->title: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->title: " + "empty." );
                    }
                }else if( lElement.getTagName().equalsIgnoreCase( "authors" ) ) {
                    NodeList lAuthorsNodes = lElement.getChildNodes();
                    lAuthors = parseAuthors( lAuthorsNodes );
                    if( lAuthors == null ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->authors: " + "corrupted." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->authors: " + "corrupted." );
                    }
                }else if( lElement.getTagName().equalsIgnoreCase( "editor" ) ) {
                    NodeList lAuthorNodes = lElement.getChildNodes();
                    lEditor = parseAuthor( lAuthorNodes );
                    if( lEditor == null ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->editor: " + "corrupted." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->editor: " + "corrupted." );
                    }
                 }else if( lElement.getTagName().equalsIgnoreCase( "reviewer" ) ) {
                    NodeList lAuthorNodes = lElement.getChildNodes();
                    lReviewer = parseAuthor( lAuthorNodes );
                    if( lReviewer == null ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->reviewer: " + "corrupted." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->reviewer: " + "corrupted." );
                    }
                } else if( lElement.getTagName().equalsIgnoreCase( "questions" ) ) {
                    NodeList lQuestionsNodes = lElement.getChildNodes();
                    lQuestions = parseQuestiones( lQuestionsNodes );
                    if( lQuestions == null ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->questions: " + "corrupted." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->questions: " + "corrupted." );
                    }
                } else if( lElement.getTagName().equalsIgnoreCase( "creation" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    /*
                     lCreation = new Date( lTextElement.getData().trim() );
                    if( lCreation == null ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->questiones: " + "corrupted." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->questiones: " + "corrupted." );
                    }
                     **/
                } else if( lElement.getTagName().equalsIgnoreCase( "publication" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    /*
                     lPublication = new Date( lTextElement.getData().trim() );
                    if( lPublication == null ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->questiones: " + "corrupted." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->questiones: " + "corrupted." );
                    }
                     **/
                } else if( lElement.getTagName().equalsIgnoreCase( "renovation" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    /*
                     lRenovation = new Date( lTextElement.getData().trim() );
                    if( lRenovation == null ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->questiones: " + "corrupted." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->questiones: " + "corrupted." );
                    }
                     **/
                }
            }
        }
        try {
            lSubject = lSubject.toUpperCase();
            lTitle = lTitle.toUpperCase();
            lDescription = new Description(lCreation,lPublication,lRenovation,lSubject,lTitle);
            lTest = new Test(
                    lAuthors,
                    lEditor,
                    lReviewer,
                    lDescription,
                    lQuestions);
        } catch ( Exception eInvalidXMLDocument ) {
            mLog.add( 
                    "Source XML file has errors near " +
                    "test: " + "corrupted." );
            throw new Exception(
                    "Source XML file has errors near " +
                    "test: " + "corrupted." );
        }
        
        return lTest;
    }
    
    private Vector <Author> parseAuthors(
            NodeList pAuthorNodes ) 
            throws Exception {
        
        Vector <Author> lAuthors = new Vector <Author>();
        Author lAuthor = null;
        
        for( int iNode = 0; iNode < pAuthorNodes.getLength(); iNode++ ) {
            Node lNode = pAuthorNodes.item( iNode );
            if( lNode instanceof Element ) {
                Element lElement = ( Element )lNode;
                if( lElement.getTagName().equalsIgnoreCase( "author" ) ) {
                    NodeList lAuthorsNode = lElement.getChildNodes();
                    lAuthor = parseAuthor( lAuthorsNode );
                    if( lAuthor == null ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->author: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->author: " + "empty." );
                    }
                    else
                    {
                        lAuthors.add(lAuthor);
                    }
                }
            }
        }
        return lAuthors;
    }
      
    private Author parseAuthor(
            NodeList pAuthorNodes ) 
            throws Exception {
        
        String lCourse = null;
        String lName = null;
        String lSurname = null;
        String lPatronymic = null;
        Author lAuthor = null;
        Department lDepartment = null;
        
        for( int iNode = 0; iNode < pAuthorNodes.getLength(); iNode++ ) {
            Node lNode = pAuthorNodes.item( iNode );
            if( lNode instanceof Element ) {
                Element lElement = ( Element )lNode;
                if( lElement.getTagName().equalsIgnoreCase( "course" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lCourse = new String( lTextElement.getData().trim() );
                    if( lCourse == null ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->author->course: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->author->course: " + "empty." );
                    }
                } else if( lElement.getTagName().equalsIgnoreCase( "name" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lName = new String( lTextElement.getData().trim() );
                    if( lName.trim().length() == 0 ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->author->name: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->author->name: " + "empty." );
                    }
                } else if( lElement.getTagName().equalsIgnoreCase( "surname" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lSurname = new String( lTextElement.getData().trim() );
                    if( lSurname.trim().length() == 0 ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->author->surname: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->author->surname: " + "empty." );
                    }
                } else if( lElement.getTagName().equalsIgnoreCase( "patronymic" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lPatronymic = new String( lTextElement.getData().trim() );
                    if( lPatronymic.trim().length() == 0 ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->author->patronymic: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->author->patronymic: " + "empty." );
                    }
                } else if( lElement.getTagName().equalsIgnoreCase( "department" ) ) {
                    NodeList lQuestionsNodes = lElement.getChildNodes();
                    lDepartment = parseDepartment( lQuestionsNodes );
                    if( lDepartment == null ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->author->department: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->author->department: " + "empty." );
                    }
                }
            }
        }
        try {
            lName = lName.toUpperCase();
            lSurname = lSurname.toUpperCase();
            lPatronymic = lPatronymic.toUpperCase();
            lAuthor = new Author(
                        lCourse,
                        lName,
                        lSurname,
                        lPatronymic,
                        lDepartment);
        } catch ( Exception eInvalidXMLDocument ) {
            mLog.add( 
                    "Source XML file has errors near " +
                    "test->author: " + "corrupted." );
            throw new Exception(
                    "Source XML file has errors near " +
                    "test->author: " + "corrupted." );
        }
        return lAuthor;
    }
    
    private Department parseDepartment(
            NodeList pDepartmentNodes ) 
            throws Exception {
        
        String lUniversity = null;
        String lFaculty = null;
        String lSubFaculty = null;
        Department lDepartment = null;

        for( int iNode = 0; iNode < pDepartmentNodes.getLength(); iNode++ ) {
            Node lNode = pDepartmentNodes.item( iNode );
            if( lNode instanceof Element ) {
                Element lElement = ( Element )lNode;
                if( lElement.getTagName().equalsIgnoreCase( "university" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lUniversity = new String( lTextElement.getData().trim() );
                    if( lUniversity.trim().length() == 0 ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->department->university: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->department->university: " + "empty." );
                    }
                } else if( lElement.getTagName().equalsIgnoreCase( "faculty" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lFaculty = new String( lTextElement.getData().trim() );
                    if( lFaculty.trim().length() == 0 ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->department->faculty: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->department->faculty: " + "empty." );
                    }
                } else if( lElement.getTagName().equalsIgnoreCase( "subfaculty" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lSubFaculty = new String( lTextElement.getData().trim() );
                    if( lSubFaculty.trim().length() == 0 ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->department->subfaculty: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->department->subfaculty: " + "empty." );
                    }
                }
            }
        }
        try {
            lUniversity = lUniversity.toUpperCase();
            lFaculty = lFaculty.toUpperCase();
            lSubFaculty = lSubFaculty.toUpperCase();
            lDepartment = new Department(
                    lUniversity,
                    lFaculty,
                    lSubFaculty );
        } catch ( Exception eInvalidXMLDocument ) {
            mLog.add( 
                    "Source XML file has errors near " +
                    "test->department: " + "corrupted." );
            throw new Exception(
                    "Source XML file has errors near " +
                    "test->department: " + "corrupted." );
        }
        return lDepartment;
    }
    
    private Vector< Question > parseQuestiones(
            NodeList pQuestionsNodes ) 
            throws Exception {

        Vector< Question > lQuestions = new Vector< Question >();
        
        for( int iNode = 0; iNode < pQuestionsNodes.getLength(); iNode++ ) {
            Node lNode = pQuestionsNodes.item( iNode );
            if( lNode instanceof Element ) {
                Element lElement = ( Element )lNode;
                if( lElement.getTagName().equalsIgnoreCase( "question" ) ) {
                    NodeList lQuestionNodes = lElement.getChildNodes();
                    Question lQuestion = parseQuestion( lQuestionNodes );
                    if( lQuestion != null ) {
                        lQuestions.add( lQuestion );
                    } else {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->questions: corrupted." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->questions: corrupted." );
                    }
                }
            }
        }
        return lQuestions;
    }
    
    private Question parseQuestion(
            NodeList pQuestionNodes ) 
            throws Exception {
        
        String lContent = null;
        String lExplanation = null;
        QuestionType lType = null;
        Integer lCost = 0;
        Integer lPenalty = 0;
        Vector< Answer > lAnswers = new Vector <Answer>();
        Question lQuestion = null;
        
        for( int iNode = 0; iNode < pQuestionNodes.getLength(); iNode++ ) {
            Node lNode = pQuestionNodes.item( iNode );
            if( lNode instanceof Element ) {
                Element lElement = ( Element )lNode;
                if( lElement.getTagName().equalsIgnoreCase( "content" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lContent = new String( lTextElement.getData().trim() );
                    if( lContent.trim().length() == 0 ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->question->content: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->question->content: " + "empty." );
                    }
                } else if( lElement.getTagName().equalsIgnoreCase( "explanation" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lExplanation = new String( lTextElement.getData().trim() );
                    // Explanation can be empty string
                    if( lExplanation == null ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->question->explanation: " + "corrupted." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->question->explanation: " + "corrupted." );
                    }
                } else if( lElement.getTagName().equalsIgnoreCase( "cost" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lCost = new Integer( lTextElement.getData().trim() );
                } else if( lElement.getTagName().equalsIgnoreCase( "penalty" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lPenalty = new Integer( lTextElement.getData().trim() );
                } else if( lElement.getTagName().equalsIgnoreCase( "answers" ) ) {
                    NodeList lAnswersNodes = lElement.getChildNodes();
                    lAnswers = parseAnswers( lAnswersNodes );
                    if (lAnswers == null || lAnswers.size() == 0) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->question->answers: " + "corrupted or empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->question->answers: " + "corrupted or empty." );
                    }
                }
            }
        }
        try {
            lQuestion = new Question(
                    lContent,
                    lExplanation,
                    lCost,
                    lPenalty,
                    lAnswers );
        } catch ( Exception eInvalidateXMLDocument ) {
            mLog.add( 
                    "Source XML file has errors near " +
                    "test->question: " + "corrupted." );
            throw new Exception(
                    "Source XML file has errors near " +
                    "test->question: " + "corrupted." );
        }
        return lQuestion;
    }
    
    private Vector <Answer> parseAnswers(
            NodeList pAnswerNodes )
            throws Exception {
        
        Vector <Answer> lAnswers = new Vector <Answer>();
        Answer lAnswer;
        
        for( int iNode = 0; iNode < pAnswerNodes.getLength(); iNode++ ) {
            Node lNode = pAnswerNodes.item( iNode );
            if( lNode instanceof Element ) {
                Element lElement = ( Element )lNode;
                if( lElement.getTagName().equalsIgnoreCase( "answer" ) ) {
                    NodeList lAnswerNodes = lElement.getChildNodes();
                    lAnswer = parseAnswer( lAnswerNodes );
                    if (lAnswer == null) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->question->answer: " + "corrupted." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->question->answer: " + "corrupted." );
                    }
                    else lAnswers.add(lAnswer);
                }
            }
        }
        return lAnswers;
    }
    
     private Answer parseAnswer(
            NodeList pAnswerNodes )
            throws Exception {
        
        String lContent = null;
        String lCorrect = null;
        Answer lAnswer = null;
        
        for( int iNode = 0; iNode < pAnswerNodes.getLength(); iNode++ ) {
            Node lNode = pAnswerNodes.item( iNode );
            if( lNode instanceof Element ) {
                Element lElement = ( Element )lNode;
                if( lElement.getTagName().equalsIgnoreCase( "content" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lContent = new String( lTextElement.getData().trim() );
                    if( lContent.trim().length() == 0 ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->answer->content: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->answer->content: " + "empty." );
                    }
                } else if( lElement.getTagName().equalsIgnoreCase( "correct" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lCorrect = new String( lTextElement.getData().trim() );
                    if( lCorrect.trim().length() == 0 ) {
                        mLog.add( 
                                "Source XML file has errors near " +
                                "test->answer->correct: " + "empty." );
                        throw new Exception(
                                "Source XML file has errors near " +
                                "test->answer->correct: " + "empty." );
                    }
                }
            }
        }
        try {
            lAnswer = new Answer(
                    lContent,
                    lCorrect );
        } catch ( Exception eInvalidateXMLDocument ) {
            mLog.add( 
                    "Source XML file has errors near " +
                    "test->question->answer: " + "corrupted." );
            throw new Exception(
                    "Source XML file has errors near " +
                    "test->question->answer: " + "corrupted." );
        }
        return lAnswer;
    }
    
    private Document mXMLDocument;
}