package writers;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;

import java.util.Vector;

public abstract class FileTestWriter 
    implements TestWriter {
    
    public FileTestWriter() {

        mDestinationFile = null;
        mEncoding = new String( "UTF-8" );
        mLog = new Vector< String >();
    }
    
    public boolean setDestination(
            Object pDestination ) {
        
        if( pDestination == null ) {
            return false;
        }
        if( pDestination instanceof File ) {
            File lDestinationFile = ( File )pDestination;
            if( ready() ) {
                if( lDestinationFile.exists() ) {
                    mDestinationFile = lDestinationFile;
                    return true;
                } else {
                    try {
                        lDestinationFile.createNewFile();
                    } catch ( Exception eFileCreationFailed ) {
                        return false;
                    }
                    mDestinationFile = lDestinationFile;
                    return true;
                }
            } else {
                mDestinationFile = lDestinationFile.getAbsoluteFile();
                return true;
            }
        } else {
            return false;
        }
    }
    
    public boolean ready() {
        
        if( mDestinationFile != null ) {
            if( mDestinationFile.exists() ) {
                return true;
            } else {
                try {
                    mDestinationFile.createNewFile();
                } catch( Exception eFileCreationFailed ) {
                    return false;
                }
                return true;
            }
        } else {
            return false;
        }
    }

    public Vector< String > getLog() {
        
        return ( Vector < String > ) mLog.clone();
    }
    
    public void setEncoding(
            String pEncoding ) {
        
        mEncoding = new String( pEncoding );
    }
    
    public String getEncoding() {
        
        return new String( mEncoding );
    }
    
    protected boolean writeEncodingSignature(
            FileOutputStream pFOStream ) {
        
        try {
            if( mEncoding.equals( "UTF-8" ) ) {
                pFOStream.write( 0xEF );
                pFOStream.write( 0xBB );
                pFOStream.write( 0xBF );
            }
            return true;
        } catch ( Exception eFOStreamCorrupted ) {
            return false;
        }
    }
    
    protected boolean prepareOStream() {
        
        if( mDestinationFile.exists() ) {
            mDestinationFile.delete();
        }
        try {
            mDestinationFile.createNewFile();
        } catch ( Exception eFileCreationFailed ) {
            return false;
        }
        try {
            mOStream = new FileOutputStream( mDestinationFile );
        } catch( FileNotFoundException eFailed ) {
            return false;
        }
        return true;
    }
    
    protected Vector< String > mLog;
    protected FileOutputStream mOStream;
    protected File mDestinationFile;
    protected String mEncoding;
}