package writers;

import tests.*;

import java.util.Vector;

public class XMLTestWriter
    extends FileTestWriter {
    
    public XMLTestWriter() {
    }
    
    public boolean write(
            Test pTest ) {
        
        if( pTest == null ) {
            return false;
        }
        if( ! prepareOStream() ) {
            return false;
        }
        
        mLog = new Vector< String >();
        
        try {
            writeEncodingSignature( mOStream );
            writeHeader();
            writeAuthors( pTest.getAuthors() );
            writeEditor(pTest.getEditor());
            writeReviewer(pTest.getReviewer());
            writeDescription(pTest.getDescription());
            writeQuestions(pTest.getQuestions() );
            writeFooter();
            return true;
        } catch ( Exception eWritingFailed ) {
            mLog.add( "Destination file writing failed." );
            mDestinationFile.delete();
            return false;
        }
    }
    
    private void writeHeader()
        throws Exception {
        
        mOStream.write( new String( "<?xml version=\"1.0\"?>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "<test xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">\r\n" ).getBytes( mEncoding ) );
    }
    
    private void writeDescription(
            Description pDescription ) 
            throws Exception {
                
        mOStream.write( new String( "\t<creation>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t</creation>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t<publication>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t</publication>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t<renovation>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t</renovation>\r\n" ).getBytes( mEncoding ) );
        
        mOStream.write( new String( "\t<subject>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t" + pDescription.GetSubject() + "\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t</subject>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t<title>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t" + pDescription.GetTitle() + "\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t</title>\r\n" ).getBytes( mEncoding ) );
    }
    
    private void writeAuthors(
            Vector <Author> pAuthors )
            throws Exception {
                
        mOStream.write(new String("\t<authors>\r\n").getBytes(mEncoding));
        for (int CurrentAuthor = 0; CurrentAuthor < pAuthors.size(); CurrentAuthor++) {
            Author pAuthor = pAuthors.get(CurrentAuthor);
            
            mOStream.write(new String("\t<author>\r\n").getBytes(mEncoding));
            writeAuthor(pAuthor);
            mOStream.write(new String("\t</author>\r\n").getBytes(mEncoding));
        }
        mOStream.write(new String("\t</authors>\r\n").getBytes(mEncoding));
    }
        
    private void writeAuthor(Author pAuthor)
        throws Exception {
        
       // Write course
       mOStream.write( new String( "\t\t<course>\r\n").getBytes( mEncoding ) );
       mOStream.write( new String( "\t\t\t" + pAuthor.getCourse() + "\r\n" ).getBytes( mEncoding ) );
       mOStream.write( new String( "\t\t</course>\r\n").getBytes( mEncoding ) );
       // Write Name
       mOStream.write( new String( "\t\t<name>\r\n").getBytes( mEncoding ) );
       mOStream.write( new String( "\t\t\t" + pAuthor.getName() + "\r\n" ).getBytes( mEncoding ) );
       mOStream.write( new String( "\t\t</name>\r\n").getBytes( mEncoding ) );
       //Write Surname
       mOStream.write( new String( "\t\t<surname>\r\n").getBytes( mEncoding ) );
       mOStream.write( new String( "\t\t\t" + pAuthor.getSurname() + "\r\n" ).getBytes( mEncoding ) );
       mOStream.write( new String( "\t\t</surname>\r\n").getBytes( mEncoding ) );
       // Write Patronymic
       mOStream.write( new String( "\t\t<patronymic>\r\n").getBytes( mEncoding ) );
       mOStream.write( new String( "\t\t\t" + pAuthor.getPatronymic() + "\r\n" ).getBytes( mEncoding ) );
       mOStream.write( new String( "\t\t</patronymic>\r\n").getBytes( mEncoding ) );
       
       writeDepartment(pAuthor.getDepartment()); 
    }
    
    private void writeDepartment(
            Department pDepartment )
            throws Exception {
        
        mOStream.write( new String( "\t\t<department>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t<university>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t\t" + pDepartment.getUniversity() + "\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t</university>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t<faculty>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t\t" + pDepartment.getFaculty() + "\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t</faculty>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t<subfaculty>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t\t" + pDepartment.getSubFaculty() + "\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t</subfaculty>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t</department>\r\n" ).getBytes( mEncoding ) );        
    }
    
    private void writeEditor(Author pAuthor)
        throws Exception {
        
        mOStream.write(new String("\t<editor>\r\n").getBytes(mEncoding));
        writeAuthor(pAuthor);
        mOStream.write(new String("\t</editor>\r\n").getBytes(mEncoding));
    }
    
    private void writeReviewer(Author pAuthor)
        throws Exception {
        
        mOStream.write(new String("\t<reviewer>\r\n").getBytes(mEncoding));
        writeAuthor(pAuthor);
        mOStream.write(new String("\t</reviewer>\r\n").getBytes(mEncoding));
    }
    
    private void writeQuestions(
            Vector< Question > pQuestions )
            throws Exception {
        mOStream.write( new String( "\t<questions>\r\n" ).getBytes( mEncoding ) );
        for( int iQuestion = 0; iQuestion < pQuestions.size(); iQuestion++ ) {
            writeQuestion( pQuestions.get( iQuestion ) );
        }
        mOStream.write( new String( "\t</questions>\r\n" ).getBytes( mEncoding ) );
    }
    
    private void writeQuestion(
            Question pQuestion ) 
            throws Exception {
        
        mOStream.write( new String( "\t\t<question>\r\n" ).getBytes( mEncoding ) );
        // Write Content
        mOStream.write( new String( "\t\t\t<content>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t\t" + pQuestion.getContent() + "\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t</content>\r\n" ).getBytes( mEncoding ) );
        // Write Explanation
        mOStream.write( new String( "\t\t\t<explanation>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t\t" + pQuestion.getExplanation() + "\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t</explanation>\r\n" ).getBytes( mEncoding ) );
        // Write Cost
        mOStream.write( new String( "\t\t\t<cost>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t\t" + pQuestion.getCost().toString() + "\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t</cost>\r\n" ).getBytes( mEncoding ) );
        // Write Penalty
        mOStream.write( new String( "\t\t\t<penalty>\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t\t" + pQuestion.getPenalty().toString() + "\r\n" ).getBytes( mEncoding ) );
        mOStream.write( new String( "\t\t\t</penalty>\r\n" ).getBytes( mEncoding ) );
                
        writeAnswers( pQuestion.getAnswers() );
        
        mOStream.write( new String( "\t\t</question>\r\n" ).getBytes( mEncoding ) );
    }
    
    private void writeAnswers(
            Vector< Answer > pAnswers )
            throws Exception {
        
        mOStream.write( new String( "\t\t<answers>\r\n" ).getBytes( mEncoding ) );
        for( int iAnswer = 0; iAnswer < pAnswers.size(); iAnswer++ ) {
            Answer lAnswer = pAnswers.get( iAnswer );
            mOStream.write( new String( "\t\t\t<answer>\r\n" ).getBytes( mEncoding ) );
            mOStream.write( new String( "\t\t\t\t<content>\r\n" ).getBytes( mEncoding ) );
            mOStream.write( new String( "\t\t\t\t\t" + lAnswer.getContent() + "\r\n" ).getBytes( mEncoding ) );
            mOStream.write( new String( "\t\t\t\t</content>\r\n" ).getBytes( mEncoding ) );
            mOStream.write( new String( "\t\t\t\t<correct>\r\n" ).getBytes( mEncoding ) );
            mOStream.write( new String( "\t\t\t\t\t" + lAnswer.getCorrect() + "\r\n" ).getBytes( mEncoding ) );
            mOStream.write( new String( "\t\t\t\t</correct>\r\n" ).getBytes( mEncoding ) );
            mOStream.write( new String( "\t\t\t</answer>\r\n" ).getBytes( mEncoding ) );
        }
        mOStream.write( new String( "\t\t</answers>\r\n" ).getBytes( mEncoding ) );
    }
    
    private void writeFooter()
        throws Exception {
        
        mOStream.write( new String( "</test>" ).getBytes( mEncoding ) );
    }
    
}